package com.qizhen.healsphere;

import cn.hutool.core.collection.CollectionUtil;
import com.qizhen.healsphere.repository.mapper.KgEdgesMapper;
import com.qizhen.healsphere.repository.mapper.KgNodesMapper;
import com.qizhen.healsphere.repository.mapper.KgSchemasMapper;
import com.qizhen.healsphere.repository.mapper.entity.KgEdges;
import com.qizhen.healsphere.repository.mapper.entity.KgNodes;
import com.qizhen.healsphere.repository.mapper.entity.KgSchemas;
import com.qizhen.healsphere.repository.neo4j.BaseNodeRepository;
import com.qizhen.healsphere.repository.neo4j.BaseRelationshipRepository;
import com.qizhen.healsphere.repository.neo4j.entity.BaseEntity;
import com.qizhen.healsphere.repository.neo4j.entity.Edge;
import com.qizhen.healsphere.service.EntityService;
import com.qizhen.healsphere.web.vo.MergeEntityVO;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RunWith(SpringRunner.class)
@ComponentScan(basePackages = {"com.qizhen.healsphere"})
@SpringBootTest
public class MergeDataTest {
    String version = "1.1";

    @Autowired
    KgNodesMapper kgNodesMapper;

    @Autowired
    KgEdgesMapper kgEdgesMapper;

    @Autowired
    KgSchemasMapper kgSchemasMapper;

    @Autowired
    EntityService entityService;

    @Autowired
    BaseNodeRepository baseNodeRepository;

    @Test
    public void mergeData() {
        String startLabel = "药品适应症";
        String endLabel = "疾病";
        List<BaseEntity> all = baseNodeRepository.getAll(startLabel);
        for(BaseEntity temp:all){
            BaseEntity baseEntity = baseNodeRepository.byName(endLabel, temp.getName());
            if(baseEntity!=null){
                MergeEntityVO mergeEntity = new MergeEntityVO();
                mergeEntity.setFirstId(temp.getId());
                mergeEntity.setFirstLabel(startLabel);
                mergeEntity.setSecondId(baseEntity.getId());
                mergeEntity.setSecondLabel(endLabel);
                mergeEntity.setNewName(baseEntity.getName());
                mergeEntity.setNewLabel(endLabel);
                entityService.mergeEntity(mergeEntity);
            }else{
                System.out.println(baseEntity.getId());
            }
        }



    }

    private void addIfNotExist(String label){
        if("就诊科室".equals(label)) {
            label = "科室";
        }
        HashMap<String, Object> columnMap = new HashMap<>();
        columnMap.put("name", label);
        List<KgSchemas> kgSchemasList = kgSchemasMapper.selectByMap(columnMap);
        if(!CollectionUtil.isEmpty(kgSchemasList)) {
            return;
        }
        KgSchemas kgSchemas = new KgSchemas();
        kgSchemas.setName(label);
        kgSchemas.setCategory(label);
        kgSchemas.setContent("name");
        kgSchemas.setVersion(version);
        kgSchemasMapper.insert(kgSchemas);
    }

    private Integer addNode(BaseEntity node) {
        try {
            String category = node.getLabel().get(0);
            if("疾病".equals(category)){
                category = "Disease";
            }else if("就诊科室".equals(category)){
                category = "Department";
            }else if("症状".equals(category)){
                category = "Symptom";
            }
            HashMap<String, Object> columnMap = new HashMap<>();
            columnMap.put("name", node.getName());
            columnMap.put("category", category);
            List<KgNodes> kgNodesList = kgNodesMapper.selectByMap(columnMap);
            if (!CollectionUtil.isEmpty(kgNodesList)) {
                return kgNodesList.get(0).getId();
            }

            KgNodes kgNodes = new KgNodes();
            kgNodes.setName(node.getName());
            kgNodes.setCategory(category);
            kgNodes.setVersion(version);
            kgNodesMapper.insert(kgNodes);
            return kgNodes.getId();
        }catch (Exception e){
            e.printStackTrace();
            return -1;
        }
    }
}
