package com.qizhen.healsphere;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.qizhen.healsphere.repository.mapper.KgEdgesMapper;
import com.qizhen.healsphere.repository.mapper.KgNodesMapper;
import com.qizhen.healsphere.repository.mapper.KgSchemasMapper;
import com.qizhen.healsphere.repository.mapper.entity.KgEdges;
import com.qizhen.healsphere.repository.mapper.entity.KgNodes;
import com.qizhen.healsphere.repository.mapper.entity.KgSchemas;
import com.qizhen.healsphere.repository.neo4j.BaseRelationshipRepository;
import com.qizhen.healsphere.repository.neo4j.entity.BaseEntity;
import com.qizhen.healsphere.repository.neo4j.entity.Edge;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.HashMap;
import java.util.List;

@RunWith(SpringRunner.class)
@ComponentScan(basePackages = {"com.qizhen.healsphere"})
@SpringBootTest
public class MigrateDataTest {
    String version = "1.1";
    @Autowired
    BaseRelationshipRepository baseRelationshipRepository;

    @Autowired
    KgNodesMapper kgNodesMapper;

    @Autowired
    KgEdgesMapper kgEdgesMapper;

    @Autowired
    KgSchemasMapper kgSchemasMapper;

    @Test
    public void migrateData() {
        String endLabels = "药品不良反应,药品商品名,药品性状,药品成份,药品英文名称";
        String[] split = endLabels.split(",");
        String startLabel = "药品";
        addIfNotExist(startLabel);
        for(String endLabel:split) {
            String relationship = startLabel + "相关" + endLabel;
            String name = "";


            List<Edge> edges = baseRelationshipRepository.triples(startLabel, relationship, "实验室检查"+endLabel);
            if (CollectionUtil.isEmpty(edges)) {
                continue;
            }
            if ("就诊科室".equals(endLabel)) {
                name = "所属科室";
                relationship = "belongs_to";
            } else if ("症状".equals(endLabel)) {
                relationship = "has_symptom";
                name = "症状";
            } else {
                name = relationship;
            }
            addIfNotExist("实验室检查"+endLabel);

            for (Edge temp : edges) {
                Integer startNodeId = addNode(temp.getStartNode());
                Integer endNodeId = addNode(temp.getEndNode());
                if (startNodeId <= 0 || endNodeId <= 0) {
                    continue;
                }
                HashMap<String, Object> columnMap = new HashMap<>();
                columnMap.put("src_id", startNodeId);
                columnMap.put("dest_id", endNodeId);
                columnMap.put("name", name);
                List<KgEdges> kgEdgesList = kgEdgesMapper.selectByMap(columnMap);
                if (CollectionUtil.isEmpty(kgEdgesList)) {
                    KgEdges kgEdges = new KgEdges();
                    kgEdges.setSrc_id(startNodeId);
                    kgEdges.setDest_id(endNodeId);
                    kgEdges.setName(name);
                    kgEdges.setCategory(relationship);
                    kgEdges.setVersion(version);
                    kgEdgesMapper.insert(kgEdges);
                }
            }
        }
    }

    private void addIfNotExist(String label){
        if("就诊科室".equals(label)) {
            label = "科室";
        }
        HashMap<String, Object> columnMap = new HashMap<>();
        columnMap.put("name", label);
        List<KgSchemas> kgSchemasList = kgSchemasMapper.selectByMap(columnMap);
        if(!CollectionUtil.isEmpty(kgSchemasList)) {
            return;
        }
        KgSchemas kgSchemas = new KgSchemas();
        kgSchemas.setName(label);
        kgSchemas.setCategory(label);
        kgSchemas.setContent("name");
        kgSchemas.setVersion(version);
        kgSchemasMapper.insert(kgSchemas);
    }

    private Integer addNode(BaseEntity node) {
        try {
            String category = node.getLabel().get(0);
            if("疾病".equals(category)){
                category = "Disease";
            }else if("就诊科室".equals(category)){
                category = "Department";
            }else if("症状".equals(category)){
                category = "Symptom";
            }
            HashMap<String, Object> columnMap = new HashMap<>();
            columnMap.put("name", node.getName());
            columnMap.put("category", category);
            List<KgNodes> kgNodesList = kgNodesMapper.selectByMap(columnMap);
            if (!CollectionUtil.isEmpty(kgNodesList)) {
                return kgNodesList.get(0).getId();
            }

            KgNodes kgNodes = new KgNodes();
            kgNodes.setName(node.getName());
            kgNodes.setCategory(category);
            kgNodes.setVersion(version);
            kgNodesMapper.insert(kgNodes);
            return kgNodes.getId();
        }catch (Exception e){
            e.printStackTrace();
            return -1;
        }
    }
}
